@@ -6,16 +6,13 @@ import (
 
 	"github.com/docker/cli/cli"
 	"github.com/docker/cli/cli/command"
-	"github.com/docker/docker/api/types/swarm"
-	"github.com/docker/docker/client"
+	"github.com/docker/cli/internal/commands"
+	"github.com/moby/moby/client"
 	"github.com/spf13/cobra"
 )
 
-// NewNodeCommand returns a cobra command for `node` subcommands
-//
-// Deprecated: Do not import commands directly. They will be removed in a future release.
-func NewNodeCommand(dockerCLI command.Cli) *cobra.Command {
-	return newNodeCommand(dockerCLI)
+func init() {
+	commands.Register(newNodeCommand)
 }
 
 // newNodeCommand returns a cobra command for `node` subcommands
@@ -29,6 +26,7 @@ func newNodeCommand(dockerCLI command.Cli) *cobra.Command {
 			"version": "1.24",
 			"swarm":   "manager",
 		},
+		DisableFlagsInUseLine: true,
 	}
 	cmd.AddCommand(
 		newDemoteCommand(dockerCLI),
@@ -47,23 +45,23 @@ func newNodeCommand(dockerCLI command.Cli) *cobra.Command {
 // the `/info` endpoint.
 func Reference(ctx context.Context, apiClient client.APIClient, ref string) (string, error) {
 	if ref == "self" {
-		info, err := apiClient.Info(ctx)
+		res, err := apiClient.Info(ctx, client.InfoOptions{})
 		if err != nil {
 			return "", err
 		}
-		if info.Swarm.NodeID == "" {
+		if res.Info.Swarm.NodeID == "" {
 			// If there's no node ID in /info, the node probably
 			// isn't a manager. Call a swarm-specific endpoint to
 			// get a more specific error message.
 			//
 			// FIXME(thaJeztah): this should not require calling a Swarm endpoint, and we could just suffice with info / ping (which has swarm status).
-			_, err = apiClient.NodeList(ctx, swarm.NodeListOptions{})
+			_, err = apiClient.NodeList(ctx, client.NodeListOptions{})
 			if err != nil {
 				return "", err
 			}
 			return "", errors.New("node ID not found in /info")
 		}
-		return info.Swarm.NodeID, nil
+		return res.Info.Swarm.NodeID, nil
 	}
 	return ref, nil
 }
